package io.tswf.minecraft.blockventure.core.inventory.generic

import groovy.transform.CompileStatic
import net.minecraft.inventory.SidedInventory
import net.minecraft.item.ItemStack
import net.minecraft.util.math.Direction

import java.util.stream.IntStream

@CompileStatic
class GenericSidedInventoryHolder extends GenericInventoryHolder implements SidedInventory {

    public static final int[] EMPTY_SLOTS = new int[0]

    Map<Direction, int[]> sideToSlots = new HashMap<>()

    CanExtractPredicate canExtractPredicate = { slot, stack, side -> true }
    CanInsertPredicate canInsertPredicate = { slot, stack, side -> true }

    GenericSidedInventoryHolder(int inventorySize, Map<Direction, int[]> sideToSlots = new HashMap<>(), String inventoryName = defaultInventoryName) {
        super(inventorySize, inventoryName)
        this.sideToSlots = sideToSlots
    }

    int[] getSlotsForSide(Direction side) {
        def slots = sideToSlots[side] ?: EMPTY_SLOTS
        if (slots.length == 0) {
            return EMPTY_SLOTS
        }

        return Arrays.copyOf(slots, slots.length)
    }

    void removeSlotsFromSide(Direction side, int... slotsToRemove) {
        def existingSlotsForSide = sideToSlots.computeIfAbsent(side) { EMPTY_SLOTS }
        def newSlotsForSide = Arrays.stream(existingSlotsForSide)
                .distinct()
                .filter { it !in slotsToRemove }
                .toArray()

        sideToSlots[side] = newSlotsForSide
    }

    void addSlotsToSide(Direction side, int... slotsToAdd) {
        def existingSlotsForSide = sideToSlots.computeIfAbsent(side) { EMPTY_SLOTS }
        def newSlotsForSide = IntStream.concat(Arrays.stream(slotsToAdd), Arrays.stream(existingSlotsForSide))
            .distinct()
            .toArray()

        sideToSlots[side] = newSlotsForSide
    }

    @Override
    int[] getAvailableSlots(Direction side) {
        return sideToSlots[side] ?: EMPTY_SLOTS
    }

    @Override
    boolean canInsert(int slot, ItemStack stack, Direction dir) {
        return canInsertPredicate?.canInsert(slot, stack, dir) ?: false
    }

    @Override
    boolean canExtract(int slot, ItemStack stack, Direction dir) {
        return canExtractPredicate?.canExtract(slot, stack, dir) ?: false
    }

    @FunctionalInterface
    @CompileStatic
    static interface CanExtractPredicate {
        boolean canExtract(int slot, ItemStack stack, Direction dir)
    }

    @FunctionalInterface
    @CompileStatic
    static interface CanInsertPredicate {
        boolean canInsert(int slot, ItemStack stack, Direction dir)
    }
}