package io.tswf.minecraft.blockventure.core.world.chunk.persist

import groovy.transform.CompileStatic
import groovy.util.logging.Slf4j
import it.unimi.dsi.fastutil.longs.LongSet
import net.minecraft.server.world.ServerWorld
import net.minecraft.util.math.ChunkPos
import net.minecraft.world.World

@Slf4j
@CompileStatic
class ChunkPersistService {

    static boolean shouldLogPersistOperations = System.findGlobalProperty("blockventure.world.chunk.persist.logs.enabled")
        .map { it.toBoolean() }
        .orElse(true)


    static void persistChunk(World world, int chunkX, int chunkZ) {
        if (world instanceof ServerWorld) {
            if (shouldLogPersistOperations && !isChunkPersisted(world, chunkX, chunkZ)) {
                log.info("Persisting chunk ${chunkX} ${chunkZ} in world ${world.dimensionKey.value}...")
            }

            world.setChunkForced(chunkX, chunkZ, true)
        }
    }

    static void freeChunk(World world, int chunkX, int chunkZ) {
        if (world instanceof ServerWorld) {
            if (shouldLogPersistOperations && isChunkPersisted(world, chunkX, chunkZ)) {
                log.info("Free chunk ${chunkX} ${chunkZ} in world ${world.dimensionKey.value}...")
            }

            world.setChunkForced(chunkX, chunkZ, false)
        }
    }

    static boolean isChunkPersisted(World world, int chunkX, int chunkZ) {
        return isChunkPersisted(world, ChunkPos.toLong(chunkX, chunkZ))
    }

    static boolean isChunkPersisted(World world, long serializedChunkPos) {
        if (world instanceof ServerWorld) {
            def positions = getSerializedChunkPositions(world)
            return positions.contains(serializedChunkPos)
        }

        return false
    }

    static LongSet getSerializedChunkPositions(ServerWorld serverWorld) {
        return serverWorld.forcedChunks
    }
}
